/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CEEvent: header file for a C++ class wrapping the Ping call

        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CEEvent.h,v 1.30 2005/10/04 09:31:54 adorigo Exp $
*/

#ifndef __CEEVENT_H__
#define __CEEVENT_H__

#include "glite/ce/monitor-client-api-c/AbsRequest.h"
#include "glite/ce/monitor-client-api-c/Topic.h"
#include <string>

#define GETEVENTRESP ns1__GetEventResponse
#define GETEVENTCALL soap_call_ns1__GetEvent
#define EVENT _ns2__Event
#define EVENT_FIELD ns2__Event
//#define TOPIC ns3__Topic 
//#define DIALECT ns3__Dialect

/// Wrapper class for getEvent request
/**
   This class is a wrapper of the getEvent remote call and hides the user from any communication and SOAP detail.\n
   It is not (yet) thread safe and can be used safely for multiple getEvent remote calls (remember to use AbsRequest::cleanup() before call it again and again)
*/
class CEEvent : public AbsRequest {
  
 private:
  TOPIC *topic;

  GETEVENTRESP evtResp;
  int sizeMessage, messageCounter;
  bool retrievedInfo;

 protected:
  CEEvent(const CEEvent&) {}
  CEEvent& operator=(const CEEvent&) { return *this; }

 public:
  /// Creates a CEEvent object
  /**
	Creates a CEEvent object and initializes the SOAP runtime for SSL authentication.
        \param certfile  The filename of the user's proxy certificate (voms or non-voms).
      	\param certpath  The name of the directory containing the certification authority's files for reverse AuthN (i.e. service authenticates itself in the client). For now can be any existing path (the reverse AuthN is not yet implemented).
	\throw AuthenticationException  If the certfile and/or certpath are missing on disk or not readable/accessible, or if the user's proxy certificate is corrupted or for some reason not accepted by gSOAP's internal SSL routines. AuthenticationException::what() gives the user the human readable reason of the exception.
  */
   CEEvent(const std::string& certfile, const std::string& certpath) throw(AuthenticationException&, GeneralException&);

  /// Destroy the CEEvent object
  /**
	Simply calls AbsRequest::DTOR that cleanly destroys the SOAP runtime
  */
  virtual ~CEEvent(void);

  /// Sets the requested event's parameter
  /**
	The CEMonitor getEvent remote call returns an event generated by a certain sensor. The specification of the sensor the user want the event of, is made by creating a Topic object and passing it as argument of this method. This method must be called before the first call of CEEvent::getEvent(), and can be called again before subsequent calls of CEEvent::getEvent().
	\sa Topic
	\param topic  The topic (sensor) generating the events the user is interested to
  */
  void setRequestParam(Topic *topic);
  /// Invokes the remote call getEvent
  /**
	Invokes the CEMonitor remote call getEvent and put the received event into an internal array of strings that the user can browse by using CEEvent::getNextEventMessage().
	\sa CEEvent::getNextEventMessage()
	\throw GenericException&  When the CEMonitor service sends back to the client a generic fault;
	\throw TopicNotSupportedException&  When the user specified an un-existing topic name;
	\throw DialectNotSupportedException&  When the user asked for an event with an unsupported dialect language;
	\throw ServiceNotFoundException&  When the user specified an un-existing service's address (see AbsRequest::setServiceURL() for the correct address format). This exception can be raised if the host is unreachable/non-existent, or if the connection is refused (e.g. wrong tcp port), or if the root-context of the remote container (TomCAT) is wrong;
	\throw GeneralException&  If some other kind of error occurred.
	All raised exceptions contain human readable information the user can get using the method ::what().
  */
  void getEvent(void) throw(GenericException&,
			    TopicNotSupportedException&,
			    DialectNotSupportedException&,
			    ServiceNotFoundException&,
			    GeneralException&) ;

  /// Gets the ID of last event
  /**
	Returns the ID (integer) of last retrieved event.
	\return The integer ID of last event
  */
  int getEventID() const;
  /// Gets the timestamp of last event
  /**
	Returns the timestamp (expressed as an integer indicating the number of second since the "epoch") of last retrieved event.
	\return The last event's timestamp
  */
  time_t getEventTimestamp() const;
  /// Gets the event's timestamp
  /**
	Returns the human readable timestamp of last retrieved event.
	\return The last event's timestamp in human readable format
  */
  const char* getEventDateTime() const;
  /// Gets the event's producer
  /**
	Returns the name of the sensor that produced last retrieved event
	\return The name of the last event producer
  */
  const char* getEventProducer() const;
  /// Returns the next event message
  /**
	This method returns the next event message stored in an array consisting of
	CEEvent::getNumberOfMessages() messages retrieved by a previous call of
	CEEvent::getEvent(). When the last message is reached, a subsequent call
	of this method returns NULL.
  */
  const char* getNextEventMessage();
  /// Returns the last event message
  /**
	Returns the last event message stored in an array consisting of
	CEEvent::getNumberOfMessages() messages retrieved by a previous call of
	CEEvent::getEvent().
  */
  const char* getLastEventMessage() const ;
  /// Returns a particular message
  /**
	Returns the message at position index from the array consisting of
	CEEvent::getNumberOfMessages() messages retrieved by a previous call of
	CEEvent::getEvent().
  */
  const char* getEventMessageAt(int index) const;
  /// Returns the number of event messages
  /**
	Returns the number of messages stored in an array consisting of
	CEEvent::getNumberOfMessages() messages retrieved by a previous call of
	CEEvent::getEvent().
  */
  int         getNumberOfMessages() const;
};

#endif
