
/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CESubscription.: header file for a C++ class wrapping the subscription (and related) call

        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CESubscription.h,v 1.26 2005/10/04 09:31:54 adorigo Exp $
*/

#ifndef __CESUBSCRIPTION_H__
#define __CESUBSCRIPTION_H__

#include "glite/ce/monitor-client-api-c/AbsRequest.h"
#include "glite/ce/monitor-client-api-c/Topic.h"
#include "glite/ce/monitor-client-api-c/Policy.h"
#include <string>

/**
 *
 * Defines macros for function calls and
 * data structures depending on the gSOAP version in use
 *
 */
#define SUBSCRIBE soap_call_ns1__Subscribe
#define UNSUBSCRIBE soap_call_ns1__Unsubscribe
#define SUBSCRIPTION _ns2__Subscription
#define SUBSCRIPTIONREF _ns2__SubscriptionRef
#define SUBSCRIBERESPONSE ns1__SubscribeResponse
#define UNSUBSCRIBERESPONSE ns1__UnsubscribeResponse
#define SUBSCRIPTIONREF_FIELD ns2__SubscriptionRef
#define PAUSESUB soap_call_ns1__PauseSubscription
#define PAUSESUBRESPONSE ns1__PauseSubscriptionResponse
#define RESUMESUBRESPONSE ns1__ResumeSubscriptionResponse
#define RESUMESUB soap_call_ns1__ResumeSubscription

/// Wrapper class for subscribe/unsubscribe/suspend/resume requests
/**
	This class is a wrapper of the all job-related remote calls and hides the user from any communication and SOAP detail.\n
	It is not (yet) thread safe and can be used safely for multiple remote calls.
*/
class CESubscription : public AbsRequest {

private:

  std::string         subscriptionID;
  TOPIC               *topic;
  POLICY              *policy;
  CESubscription(const CESubscription&);
  CESubscription& operator=(const CESubscription&);
  std::string consumerURL;
  time_t duration;

public:

  /// Creates a CESubscription object
  /**
	Creates a CESubscription object and initializes the SOAP runtime for SSL authentication.
        \param certfile  The filename of the user's proxy certificate (voms or non-voms).
      	\param certpath  The name of the directory containing the certification authority's files for reverse AuthN (i.e. service authenticates itself in the client). For now can be any existing path (the reverse AuthN is not yet implemented).
	\throw AuthenticationException  If the certfile and/or certpath are missing on disk or not readable/accessible, or if the user's proxy certificate is corrupted or for some reason not accepted by gSOAP's internal SSL routines. AuthenticationException::what() gives the user the human readable reason of the exception.
  */
  CESubscription(const std::string& certfile, const std::string& certpath) throw(AuthenticationException&, GeneralException&);
  /// Destroy the CESubscription object
  /**
	Simply calls AbsRequest::DTOR that cleanly destroys the SOAP runtime
  */
  virtual ~CESubscription();

  /// Subscribes a consumer
  /**
	This method invokes the remote call Subscribe on the remote CEMonitor and must be called AFTER calling   AbsRequest::setServiceURL() and CESubscription::setSubscribeParam(). After a successfull call of subscribe() (i.e. no exception has been raised) the user can get the CEMonitor's response (the subscription ID) by calling CESubscription::getSubscriptionID().
	\throw GenericException&  When the CEMonitor service sends back to the client a generic fault;
	\throw TopicNotSupportedException&  When the user tried to subscribe a consumer to an un-existing topic;
	\throw DialectNotSupportedException&  When the user tried to subscribe a consumer to a topic specifying an unsupported dialect language;
	\throw ServiceNotFoundException&  When the user specified an un-existing service's address (see AbsRequest::setServiceURL() for the correct address format). This exception can be raised if the host is unreachable/non-existent, or if the connection is refused (e.g. wrong tcp port), or if the root-context of the remote container (TomCAT) is wrong;
	\throw SubscriptionException&  If the subscription failed for some other reason;
	\throw GeneralException&  If some other kind of error occurred.
	All raised exceptions contain human readable information the user can get using proper methods (see AbsException and CEException class documentation).
  */
  void        subscribe() throw(AbsException&, CEException&);
  /// Removes a subscription
  /**
	This method invokes the remote call Unsubscribe on the remote CEMonitor and must be called AFTER calling   AbsRequest::setServiceURL(). A successfull call of unsubscribe() produces no response and no exception.
	\param SubscriptionID  The identifier of the subscription to remove (as returned by the Subscribe remote call)
	\throw GenericException&  When the CEMonitor service sends back to the client a generic fault;
	\throw TopicNotSupportedException&  When the user tried to subscribe a consumer to an un-existing topic;
	\throw DialectNotSupportedException&  When the user tried to subscribe a consumer to a topic specifying an unsupported dialect language;
	\throw ServiceNotFoundException&  When the user specified an un-existing service's address (see AbsRequest::setServiceURL() for the correct address format). This exception can be raised if the host is unreachable/non-existent, or if the connection is refused (e.g. wrong tcp port), or if the root-context of the remote container (TomCAT) is wrong;
	\throw SubscriptionException&  If the subscription failed for some other reason;
	\throw GeneralException&  If some other kind of error occurred.
	\sa  CESubscription::subscribe()
	All raised exceptions contain human readable information the user can get using the proper methods (see AbsException and CEException class documentation).
  */
  void        unsubscribe(const std::string& SubscriptionID) throw(GenericException&,
						    TopicNotSupportedException&, 
						    DialectNotSupportedException&, 
						    SubscriptionException&,
						    ServiceNotFoundException&, 
						    GeneralException&);
  /// Pauses a subscription
  /**
	This method invokes the remote call Pause on the remote CEMonitor and must be called AFTER calling   AbsRequest::setServiceURL(). A successfull call of pause() produces no response and no exception.
	\param SubscriptionID  The identifier of the subscription to pause (as returned by the Subscribe remote call)
	\throw GenericException&  When the CEMonitor service sends back to the client a generic fault;
	\throw TopicNotSupportedException&  When the user tried to subscribe a consumer to an un-existing topic;
	\throw DialectNotSupportedException&  When the user tried to subscribe a consumer to a topic specifying an unsupported dialect language;
	\throw ServiceNotFoundException&  When the user specified an un-existing service's address (see AbsRequest::setServiceURL() for the correct address format). This exception can be raised if the host is unreachable/non-existent, or if the connection is refused (e.g. wrong tcp port), or if the root-context of the remote container (TomCAT) is wrong;
	\throw SubscriptionException&  If the subscription failed for some other reason;
	\throw GeneralException&  If some other kind of error occurred.
	\sa  CESubscription::subscribe()
	All raised exceptions contain human readable information the user can get using the proper methods (see AbsException and CEException class documentation).
  */
  void  pause(const std::string& SubscriptionID) throw(GenericException&,
					      TopicNotSupportedException&, 
					      DialectNotSupportedException&, 
					      SubscriptionException&,
					      ServiceNotFoundException&, 
					      GeneralException&);
  /// Resumes a previously paused subscription
  /**
	AbsRequest::setServiceURL(). A successfull call of resume() produces no response and no exception.
	\param SubscriptionID  The identifier of the subscription to resume (as returned by the Subscribe remote call)
	\throw GenericException&  When the CEMonitor service sends back to the client a generic fault;
	\throw TopicNotSupportedException&  When the user tried to subscribe a consumer to an un-existing topic;
	\throw DialectNotSupportedException&  When the user tried to subscribe a consumer to a topic specifying an unsupported dialect language;
	\throw ServiceNotFoundException&  When the user specified an un-existing service's address (see AbsRequest::setServiceURL() for the correct address format). This exception can be raised if the host is unreachable/non-existent, or if the connection is refused (e.g. wrong tcp port), or if the root-context of the remote container (TomCAT) is wrong;
	\throw SubscriptionException&  If the subscription failed for some other reason;
	\throw GeneralException&  If some other kind of error occurred.
	\sa  CESubscription::subscribe()
	All raised exceptions contain human readable information the user can get using the methods (see AbsException and CEException class documentation).
  */
  void        resume(const std::string& SubscriptionID) throw(GenericException&,
					       TopicNotSupportedException&, 
					       DialectNotSupportedException&,
					       SubscriptionException&, 
					       ServiceNotFoundException&, 
					       GeneralException&);
  /// Returns the subscription identifier.
  /**
	This method has to be called after a successfull call of CESubscription::subscribe()
	\return A string representing a unique ID for the subscription done by CESubscription::subscribe()
  */
  std::string getSubscriptionID() const;
  /// Set the subscription parameters
  /**
	Sets subscription parameters.
	\param consumerURL the address of the notification consumer (must have the format http://[host]:[port]);
	\param topic the Topic that will send events to consumer (\see Topic);
	\param policy the Policy that rules the action the CEMonitor takes when an event happens (\see Policy);
	\param duration the subscription duration (in seconds).
  */
  void        setSubscribeParam(const std::string& consumerURL,
			      const Topic& topic,
			      const Policy& policy,
			      const long& duration
			      );
};

#endif
